"""DO NOT MODIFY. Auto-generated by script/fingerprint_frontend."""

FINGERPRINTS = {
    "compatibility.js": "8e4c44b5f4288cc48ec1ba94a9bec812",
    "core.js": "d4a7cb8c80c62b536764e0e81385f6aa",
    "frontend.html": "bdcde4695ce32595a9e1d813b9d7c5f9",
    "mdi.html": "c92bd28c434865d6cabb34cd3c0a3e4c",
    "micromarkdown-js.html": "93b5ec4016f0bba585521cf4d18dec1a",
    "panels/ha-panel-automation.html": "4f98839bb082885657bbcd0ac04fc680",
    "panels/ha-panel-config.html": "76853de505d173e82249bf605eb73505",
    "panels/ha-panel-dev-event.html": "4886c821235492b1b92739b580d21c61",
    "panels/ha-panel-dev-info.html": "24e888ec7a8acd0c395b34396e9001bc",
    "panels/ha-panel-dev-service.html": "92c6be30b1af95791d5a6429df505852",
    "panels/ha-panel-dev-state.html": "8f1a27c04db6329d31cfcc7d0d6a0869",
    "panels/ha-panel-dev-template.html": "d33a55b937b50cdfe8b6fae81f70a139",
    "panels/ha-panel-hassio.html": "9474ba65077371622f21ed9a30cf5229",
    "panels/ha-panel-history.html": "35177e2046c9a4191c8f51f8160255ce",
    "panels/ha-panel-iframe.html": "d920f0aa3c903680f2f8795e2255daab",
    "panels/ha-panel-logbook.html": "7c45bd41c146ec38b9938b8a5188bb0d",
    "panels/ha-panel-map.html": "0ba605729197c4724ecc7310b08f7050",
    "panels/ha-panel-zwave.html": "2ea2223339d1d2faff478751c2927d11",
    "websocket_test.html": "575de64b431fe11c3785bf96d7813450"
}
